'use strict';

// CONTROLLERS
angular.module(_CONTROLLERS_, [])
	// Root controller: Used for initial setup and global event sharing
	.controller('IndexCtrl', ['$scope', '$location', '$rootScope', '$window', 'toaster', '$timeout', 'authHelper', '$localStorage', 'snapRemote', 'UAService', 'AUTH_EVENTS', 'susy',
		function ($scope, $location, $rootScope, $window, toaster, $timeout, authHelper, $localStorage, snapRemote, UAService, AUTH_EVENTS, susy) {

			// Create app
			$rootScope.app = {
				toast: {},
				user: {
					authenticated: authHelper.isAuthenticated(),
					mobile: ''
				},
				loading: true,
				showBackBtn: false
			};

			snapRemote.getSnapper().then(function (snapper) {
				$rootScope.snapper = snapper;
			});

			$scope.snapDrawerOptions = {
				addBodyClasses: true,
				maxPosition: 280,
				minPosition: 0,
				disable: 'right'
			};

			if (_ENV_ === 'phonegap') {
				$timeout(function () {
					//$window.navigator.splashscreen && $window.navigator.splashscreen.hide();
					$window.navigator.splashscreen.hide();
				}, 1000);

				if (UAService.android) {
					$rootScope.app.UA = 'android';
					angular.element($window.document.documentElement).addClass('os-android');
				} else if (UAService.iOS) {
					$rootScope.app.UA = 'iOS';
				}
			}

			// Listen for notAuthenticated event and redirect to /login page 
			$rootScope.$on(AUTH_EVENTS.notAuthenticated, function () {
				$rootScope.app.user.authenticated = false;
				$rootScope.app.route = '/login';
			});

			$rootScope.$on('$locationChangeStart', function (event, next, prev) {
				var path = $location.path();
				if (!authHelper.isAuthenticated() && path != '/login' && path != '/smscode') {
					event.preventDefault();
					$rootScope.app.route = '/login';
				}
			});

			$rootScope.direction = 'ltr';
			$rootScope.$on('$routeChangeStart', function (event, next, prev, current) {
				$rootScope.app.showBackBtn = false;
				$rootScope.app.showQrCodeHeaderIcon = false;
				if (next.$$route.requireLogin && !authHelper.isAuthenticated()) {
					$rootScope.$broadcast(AUTH_EVENTS.notAuthenticated);
					return;
				}

				snapRemote.close();
				$(".snap-content").width($(window).width());
				$(".snap-content").removeAttr("style");

				$rootScope.pageTitle = '';

				$rootScope.direction = 'rtl';
				if (prev && next && (prev.$$route.depth > next.$$route.depth)) {
					$rootScope.direction = 'ltr';
				}
			});

			// On app.route change clear messages and navigate to the given page
			$rootScope.$watch('app.route', function (newRoute, oldRoute) {
				$rootScope.app.toast.show = false;
				$rootScope.app.loading = false;
				$location.path(newRoute);
			});

			// Logout function
			$rootScope.logout = function () {
				authHelper.logout();
				$rootScope.app.user.authenticated = false;
				$rootScope.app.route = '/login';
			};

			var connection = function (e) {
				if (e.type === 'offline') {
					$timeout(function () {
						toaster.show('Ingen dataforbindelse', false, 'notice');
					}, 100);
				}
				if (e.type === 'online') {
					$timeout(function () {
						$rootScope.app.toast.show = false;
					}, 100);
				}
			};

			$scope.displayQrCode = function () {
				$('.flip-card').toggleClass('flip');
			}

			$rootScope.$on("displayQrCode", $scope.toggleQRcode);

			$window.document.addEventListener('online', connection, false);
			$window.document.addEventListener('offline', connection, false);
			//$window.document.addEventListener('resume', logger, false);
			//$window.document.addEventListener('pause', logger, false);

			var setScopeData = function (pages) {
				if (!pages) {
					console.warn("Page not found", { pageName: $routeParams.pageId });
					return;
				}

				$rootScope.app.pages = {
					TravelRules: _.find(pages, { 'TextPageId': 'TravelRules' }),
					PtoContacts: _.find(pages, { 'TextPageId': 'PtoContacts' }),
				};
			};

			if ($localStorage.pages) {
				setScopeData($localStorage.pages);
			}
		}])
	.controller('LoginCtrl', ['$scope', '$location', '$rootScope', '$window', 'toaster', '$timeout', '$http', 'authHelper', 'susy',
		function ($scope, $location, $rootScope, $window, toaster, $timeout, $http, authHelper, susy) {

			$rootScope.snapper.disable();

			$scope.displayValidation = function (form) {
				return (form.apiError) || (form.cpr.$dirty && form.cpr.$error.required) || (form.cpr.$dirty && form.cpr.$error.minlength) || (form.mobile.$dirty && form.mobile.$error.required) || (form.mobile.$dirty && form.mobile.$error.minlength);
			}

			$scope.login = function () {
				// Verify valid form
				if ($scope.form.$invalid) {
					var errors = [];

					if ($scope.form.cpr.$error.required) errors.push("Indtast dit CPR-nummer.");
					if ($scope.form.cpr.$dirty && $scope.form.cpr.$error.minlength) errors.push("Indtast gyldigt CPR-nummer på 10 cifre.");
					if ($scope.form.mobile.$error.required) errors.push("Indtast dit mobilnummer.");
					if ($scope.form.mobile.$dirty && $scope.form.mobile.$error.minlength) errors.push("Indtast gyldigt mobilnummer på minimum otte cifre.");

					var errorMsg = errors.join("\n");
					navigator.notification.alert(errorMsg, null, "Fejl i indtastning", "OK");
					return;
				}

				// Verify CPR and mobile in Susy
				$rootScope.app.loading = true;
				susy.verifyCprAndMobile($scope.cpr, $scope.mobile)
				.then(function () {
					$rootScope.app.user.mobile = $scope.mobile;
					$rootScope.app.route = '/smscode';
				})
				.catch(function (errorMsg) {
					navigator.notification.alert(errorMsg, null, "Mit Ungdomskort", "OK");
				})
				.finally(function () {
					$rootScope.app.loading = false;
				});
			};
		}])
	.controller('CodeCtrl', ['$scope', '$location', '$rootScope', '$window', 'toaster', '$timeout', 'authHelper', 'susy',
		function ($scope, $location, $rootScope, $window, toaster, $timeout, authHelper, susy) {

			$rootScope.snapper.disable();

			$scope.login = function () {
				$scope.form.submitted = true;

				// Verify valid form
				if ($scope.form.$invalid) {
					var errors = [];

					if ($scope.form.code.$error.required) errors.push("Indtast kode modtaget på SMS.");

					var errorMsg = errors.join("\n");
					navigator.notification.alert(errorMsg, null, "Fejl i indtastning", "OK");
					return;
				}

				// Verify SMS code in Susy
				$rootScope.app.loading = true;

				susy.verifySmsCode($rootScope.app.user.mobile, $scope.code)
				.then(function (data) {
					// Save credentials and redirect to index
					authHelper.saveCredentials(data.token, $rootScope.app.user.mobile);
					$rootScope.app.route = '/';
					$rootScope.app.user.authenticated = true;
					$location.replace();
					$rootScope.snapper.enable();
				})
				.catch(function (errorMsg) {
					navigator.notification.alert(errorMsg, null, "Mit Ungdomskort", "OK");
				})
				.finally(function () {
					$rootScope.app.loading = false;
				});
			};
		}])
	.controller('CardsCtrl', ['$scope', '$location', '$rootScope', '$window', 'toaster', '$timeout', '$localStorage', 'authHelper', 'notifications', 'susy',
		function ($scope, $location, $rootScope, $window, toaster, $timeout, $localStorage, authHelper, notifications, susy) {
			$rootScope.snapper.enable();

			$scope.goToCard = function (path) {
				if ($rootScope.app.route == path)
					$window.history.forward();
				else {
					$rootScope.app.route = path;
				}
			};

			var setScopeData = function (pages) {
				if (!pages) {
					console.warn("Page not found", { pageName: $routeParams.pageId });
					return;
				}

				$rootScope.app.pages = {
					TravelRules: _.find(pages, { 'TextPageId': 'TravelRules' }),
					PtoContacts: _.find(pages, { 'TextPageId': 'PtoContacts' }),
				};
			};

			var refreshTextPages = function () {
				susy.getTextPages()
				.then(function (data) {
					$localStorage.pages = data;
					setScopeData($localStorage.pages);
				})
				.catch(function (error) {
					if (error.unauthorized) {
						console.warn("Unauthenticated for getTextPages()", error);
						authHelper.logout();
						$rootScope.app.route = '/';
					} else {
						toaster.show(error, 1000);
					}
				})
				.finally(function () {
					$rootScope.app.loading = false;
				})
			}

			var refreshCards = function () {
				// Load cards from Susy
				susy.getCards()
				.then(function (data) {
					// Save to local storage
					$localStorage.cards = data;

					bindCardsToScope(data);
					if (data) {
						console.info("Cards data refreshed", data);
					}
				})
				.catch(function (error) {
					if (error.unauthorized) {
						console.warn("Unauthenticated for getCards()", error);
						authHelper.logout();
						$rootScope.app.route = '/';
					} else {
						toaster.show(error, 1000);
					}
				})
				.finally(function () {
					$rootScope.app.loading = false;
				});
			};

			var bindCardsToScope = function (cards) {
				if (cards) {
					$scope.lastUpdated = cards.Timestamp;

					$scope.cards = {
						CurrentCards: cards.CurrentCards,
						NextCards: cards.NextCards,
						InactiveCards: cards.InactiveCards
					};
				}
			};

			var cards = $localStorage.cards;
			if(cards){
				bindCardsToScope(cards);
			}

			//Check for data refresh
			if (!cards || susy.needsDataRefresh(cards.Timestamp)) {
				refreshCards();
			}
			refreshTextPages();
		}])
	.controller('DetailCtrl', ['$rootScope', '$scope', '$routeParams', 'toaster', '$localStorage', 'susy',
		function ($rootScope, $scope, $routeParams, toaster, $localStorage, susy) {
			$rootScope.snapper.enable();
			$rootScope.app.showQrCodeHeaderIcon = true;
			$scope.calcClass = function (card) {
				if (!card) return 'expired';
				return card.Status;
			};

			$rootScope.app.showBackBtn = true;
			var cards = $localStorage.cards;
			var card;

			_.forEach(cards, function (obj) {
				if (!card) {
					card = _.find(obj, { 'Id': $routeParams.cardId });
				}
			});

			// Check for card
			if (!card) {
				console.warn("Card not found", { cardType: $routeParams.cardId });
				$rootScope.app.route = '/';
				return;
			}

			// Log to Susy - fire and forget
			if (card) {
				susy.logCardShown(card.Id)
				.then(function () {
					console.log("Logged display of card to Susy", { cardId: card.Id });
				}).catch(function (data) {
					console.warn("Failed to log display of card to Susy", { cardId: card.Id, error: data });
				});
			}

			// Set scope
			$scope.card = card;
			$scope.lastUpdated = cards.Timestamp;
			$scope.photo = card.Photo != null ? ("data:image/jpeg;base64," + card.Photo.replace(/\"/g, '')) : null;
			$scope.baseUrl = _WEB_;
			// If route info is one line only, then apply bigger formatting
			$scope.formatRouteBig = (card.RouteData1 != null 
									 && card.RouteHeading1 == null
									 && card.RouteHeading2 == null
									 && card.RouteHeading3 == null
									 && card.RouteData2 == null 
									 && card.RouteData3 == null);

			$scope.qrcodeView = false;
		}])
	// Infomation controller: Used to display static pages
	.controller('InfoCtrl', ['$rootScope', '$scope', '$routeParams', 'console', '$localStorage', 'susy',
		function ($rootScope, $scope, $routeParams, console, $localStorage, susy) {

			$rootScope.snapper.enable();

			var setScopeData = function (pages) {
				var page = _.find(pages, { 'TextPageId': $routeParams.pageId });
				if (!page) {
					console.warn("Page not found", { pageName: $routeParams.pageId });
					$rootScope.app.route = '/';
					return;
				}

				$scope.page = page;
				$scope.pageTitle = $scope.page.Headline;
				$scope.content = $scope.page.Text;
			};

			var reloadPageData = function () {
				$rootScope.app.loading = true;
				susy.getTextPages()
				.then(function (data) {
					$localStorage.pages = data;
					setScopeData($localStorage.pages);
				})
				.finally(function () {
					$rootScope.app.loading = false;
				});
			};

			// Lookup pages in local storage
			if ($localStorage.pages) {
				setScopeData($localStorage.pages);
			} else {
				reloadPageData();
			}
		}]);
