'use strict';

// SERVICES
angular.module(_SERVICES_, [])
	.value('version', 'v.' + _VERSION_)
	.constant('AUTH_EVENTS', {
		loginSuccess: 'auth-login-success',
		loginFailed: 'auth-login-failed',
		logoutSuccess: 'auth-logout-success',
		notAuthenticated: 'auth-not-authenticated',
	})
	.factory('toaster', function ($window, $rootScope, $timeout) {
		var watch = function (newValue, oldValue) {
			if (newValue && !oldValue) {
				$timeout(function () {
					$rootScope.app.toast.show = false;
				}, 6000);
			}
		};
		var show = function (txt, timer, type) {
			if (_.isString(type)) {
				$rootScope.app.toast.type = type;
			}
			$rootScope.app.toast.text = txt;
			$rootScope.app.toast.show = true;
			if (timer) {
				$timeout(function () {
					$rootScope.app.toast.show = false;
				}, timer);
			} else {
				angular.element($window.document.getElementById('toast-msg'))
				.one('click', function () {
					$rootScope.app.toast.show = false;
					$rootScope.$apply();
				});
			}
		};
		return {
			startListening: function () {
				$rootScope.$watch('app.toast.show', watch);
			},
			show: show
		};
	})
	.factory('susy', ['$q', '$http', '$rootScope', '$timeout', '$interval', 'toaster', 'console', 'authHelper',
		function ($q, $http, $rootScope, $timeout, $interval, toaster, console, authHelper) {

			var newShortUserAgent = function () {
				return navigator.userAgent.match(/Android/i)
					|| navigator.userAgent.match(/iPhone/i)
					|| navigator.userAgent.match(/iPad/i)
					|| navigator.userAgent.match(/iPod/i)
					|| navigator.userAgent.match(/IEMobile/i)
			}

			var generateUrlWithCredentials = function (path) {
				var token = authHelper.getAuthToken();
				return _WEBSERVICE_ + path + "/" + token;
			};

			var verifyCprAndMobile = function (cpr, mobile) {
				var url = _WEBSERVICE_ + "/VerifyCprAndMobile";
				var data = { "Cpr": cpr, "Mobile": mobile };
				var deferred = $q.defer();

				$http({
					url: url,
					method: 'POST',
					data: data
				}).error(function (data, status, headers, config) {
					switch (status) {
						case 404:
							deferred.reject("Applikationen kunne ikke få forbindelse til Rejsekort.");
							break;
						default:
							deferred.reject("Der opstod en fejl. Kan ikke verificere CPR og mobilnummer. Http-status: " + status);
							break;
					}
				})
				.success(function (data, status, headers, config) {
					if (data.Success) {
						deferred.resolve();
					} else {
						switch (data.StatusText) {
							case "UnknownCpr":
								deferred.reject("Det indtastede CPR kunne ikke godkendes.");
								break;
							case "UnknownCprOrAppNotActivated":
								deferred.reject("Du har ikke aktiveret levering af periodekort på app.");
								break;
							case "NotRegisteredPhoneNumber":
								deferred.reject("Mobilnummeret stemmer ikke overens med det registerede i Mit Ungdomskort.");
								break;
							default:
								deferred.reject("Der opstod en fejl. Kan ikke verificere CPR og mobilnummer.");
								break;
						}
					}
				});

				return deferred.promise;
			};

			var verifySmsCode = function (mobile, code) {
				var url = _WEBSERVICE_ + "/VerifySmsCode";
				var data = { "Mobile": mobile, "Code": code };
				var deferred = $q.defer();

				$http({
					url: url,
					method: 'POST',
					data: data
				}).error(function (data, status, headers, config) {
					deferred.reject("Der opstod en fejl. Kan ikke verificere SMS-kode.");
				})
				.success(function (data) {
					if (data.Success) {
						deferred.resolve({ token: data.Token });
					} else {
						switch (data.StatusText) {
							case "IncorrectCode":
								deferred.reject("Det indtastede kode er ikke korrekt.");
								break;
							default:
								deferred.reject("Der opstod en fejl. Kan ikke verificere SMS-kode.");
								break;
						}
					}
				});

				return deferred.promise;
			};

			var getCards = function () {
				var url = generateUrlWithCredentials("/GetCards");
				var deferred = $q.defer();

				// Send GET request
				$http({
					url: url,
					method: 'GET',
					headers: {
						'X-App-Deviceinformation': newShortUserAgent()
					}
				}).error(function (data, status, headers, config) {
					if (status == 401) {
						deferred.reject({ unauthorized: true });
					} else {
						deferred.reject("Der opstod en fejl. Kan ikke hente periodekort.");
					}
				})
				.success(function (data) {
					deferred.resolve(data);
				});

				return deferred.promise;
			};

			var getTextPages = function () {
				var url = generateUrlWithCredentials("/GetTextPages");
				var deferred = $q.defer();

				// Send GET request
				$http({
					url: url,
					method: 'GET',
					headers: {
						'X-App-Deviceinformation': newShortUserAgent()
					}
				}).error(function (data, status, headers, config) {
					if (status == 401) {
						deferred.reject({ unauthorized: true });
					} else {
						deferred.reject("Der opstod en fejl. Kan ikke hente tekstindhold.");
					}
				})
				.success(function (data) {
					deferred.resolve(data);
				});

				return deferred.promise;
			};

			var needsDataRefresh = function (lastUpdated) {
				if (!lastUpdated) return true;

				if (angular.isString(lastUpdated)) {
					lastUpdated = new Date(lastUpdated);
				}
				
				var oneHourAgo = new Date().addHours(-1);
				return (oneHourAgo > lastUpdated);
			};

			var logCardShown = function (cardId) {
				var url = generateUrlWithCredentials("/LogCardShown");

				return $http({
					url: url,
					method: 'POST',
					data: { CardId: cardId },
					headers: {
						'X-App-Deviceinformation': newShortUserAgent()
					}
				});
			};

			var getBeneficiaryPhoto = function (photoId) {
				var url = generateUrlWithCredentials("/GetBeneficiaryPhoto") + "?photoId=" + photoId;

				var deferred = $q.defer();

				// Send GET request
				$http({
					url: url,
					method: 'GET',
					headers: {
						'X-App-Deviceinformation': newShortUserAgent()
					}
				}).error(function (data, status, headers, config) {
					if (status == 401) {
						deferred.reject({ unauthorized: true });
					} else {
						deferred.reject("Der opstod en fejl. Kan ikke hente billede.");
					}
				})
				.success(function (data, status, headers, config) {
					var imageUrl = "data:image/jpeg;base64," + data.replace(/\"/g, '');
					deferred.resolve(imageUrl);
				});

				return deferred.promise;
			};

			return {
				verifyCprAndMobile: verifyCprAndMobile,
				verifySmsCode: verifySmsCode,
				getCards: getCards,
				getTextPages: getTextPages,
				needsDataRefresh: needsDataRefresh,
				logCardShown: logCardShown,
				getPhoto: getBeneficiaryPhoto
			};
		}])
	.factory('UAService', function () {
		return {
			android: navigator.userAgent.match(/Android/i),
			iOS: navigator.userAgent.match(/iPhone|iPad|iPod/i),
			windows: navigator.userAgent.match(/IEMobile/i)
		};
	})
	.factory('authHelper', ['$rootScope', '$q', '$http', '$localStorage', 'AUTH_EVENTS',
		function ($rootScope, $q, $http, $localStorage, AUTH_EVENTS) {

			// base64 encoding from
			// https://developer.mozilla.org/en-US/docs/Web/API/window.btoa?redirectlocale=en-US&redirectslug=DOM%2Fwindow.btoa
			var utf8_to_b64 = function (str) {
				return window.btoa(window.unescape(encodeURIComponent(str)));
			};

			var getAuthToken = function () {
				return $localStorage.authtoken || null;
			};

			var isAuthenticated = function () {
				return !!getAuthToken();
			};

			var saveCredentials = function (token, mobile) {
				$localStorage.authtoken = token;
				$localStorage.mobile = mobile;
				$rootScope.$broadcast(AUTH_EVENTS.loginSuccess);
			};

			var logout = function () {
				delete $localStorage.authtoken;
				delete $localStorage.mobile;
				delete $localStorage.cards;
				$rootScope.$broadcast(AUTH_EVENTS.logoutSuccess);
			};

			return {
				logout: logout,
				saveCredentials: saveCredentials,
				isAuthenticated: isAuthenticated,
				getAuthToken: getAuthToken
			};
		}
	])
	.factory('notifications', ['$rootScope', '$q',
		function ($rootScope, $q) {

			var setCardExpirationNotification = function (expirationDate) {
				var deferred = $q.defer();

				// Make sure we run in Phonegap
				if (!window.plugin || !window.plugin.notification) {
					deferred.resolve();
					return deferred.promise;
				}

				if (!expirationDate) {
					deferred.reject("No expiration date set");
					return deferred.promise;
				}

				// Cancel existing notifications				
				window.plugin.notification.local.cancelAll(function () {

					// Create new notification
					var id = window.plugin.notification.local.add({
						id: '1',  // Must be a number on Android
						date: expirationDate,    // This expects a date object
						message: 'Dit periodekort udløber snart',
						title: 'Mit Ungdomskort',
						//repeat: String,  // Either 'secondly', 'minutely', 'hourly', 'daily', 'weekly', 'monthly' or 'yearly'
						//json: String,  // Data to be passed through the notification
						autoCancel: true, // Setting this flag and the notification is automatically canceled when the user clicks it
						ongoing: false, // Prevent clearing of notification (Android only)
					});

					deferred.resolve({ notificationId: id });
				});

				return deferred.promise;
			};

			return {
				setCardExpirationNotification: setCardExpirationNotification
			};
		}
	]);
